@@ -429,8 +429,9 @@ def AddScaleToZeroArgs(parser, version):
         type=arg_parsers.Duration(lower_bound='300s', upper_bound='28800s'),
         hidden=True,
         help="""\
-Duration (in seconds) before a deployment is enrolled in scale to zero
-evaluation after scaling up. Defaults to 1 hour if min replica count is 0.
+Minimum duration (in seconds) that a deployment will be scaled up before traffic
+is evaluated for potential scale-down.
+Defaults to 1 hour if min replica count is 0.
 """,
     ).AddToParser(parser)
 
@@ -550,6 +551,16 @@ https://cloud.google.com/kubernetes-engine/docs/concepts/tpus#topology.
         help="""\
 The number of nodes per replica for multihost GPU deployments. Required for
 multihost GPU deployments.
+""",
+    ).AddToParser(parser)
+    base.Argument(
+        '--gpu-partition-size',
+        type=str,
+        hidden=True,
+        help="""\
+The partition size of the GPU accelerator. This can be used to partition a
+single GPU into multiple smaller GPU instances.
+See https://cloud.google.com/kubernetes-engine/docs/how-to/gpus-multi#multi-instance_gpu_partitions for more details.
 """,
     ).AddToParser(parser)
 
