@@ -66,9 +66,11 @@ from googlecloudsdk.core import metrics
 from googlecloudsdk.core import properties
 from googlecloudsdk.core import resources
 from googlecloudsdk.core.console import progress_tracker
+from googlecloudsdk.core.universe_descriptor import universe_descriptor
 from googlecloudsdk.core.util import retry
 import six
 
+
 DEFAULT_ENDPOINT_VERSION = 'v1'
 
 _NONCE_LENGTH = 10
@@ -797,7 +799,7 @@ class ServerlessOperations(object):
       tracker=None,
       asyn=False,
       allow_unauthenticated=None,
-      allow_unauth_regions=None,
+      multiregion_regions=None,
       for_replace=False,
       prefetch=False,
       build_image=None,
@@ -811,7 +813,7 @@ class ServerlessOperations(object):
       delegate_builds=False,
       base_image=None,
       build_service_account=None,
-      build_from_source_container_name='',
+      deploy_from_source_container_name='',
       build_worker_pool=None,
       build_machine_type=None,
       build_env_vars=None,
@@ -837,8 +839,7 @@ class ServerlessOperations(object):
         which should also have its IAM policy set to allow unauthenticated
         access. False if removing the IAM policy to allow unauthenticated access
         from a service.
-      allow_unauth_regions: str, For multi-region services, the regions in which
-        we will allow unauthenticated access.
+      multiregion_regions: The regions for a multi-region service.
       for_replace: bool, If the change is for a replacing the service from a
         YAML specification.
       prefetch: the service, pre-fetched for ReleaseService. `False` indicates
@@ -858,8 +859,9 @@ class ServerlessOperations(object):
       delegate_builds: bool. If true, use the Build API to submit builds.
       base_image: The build base image to opt-in automatic build image updates.
       build_service_account: The service account to use to execute the build.
-      build_from_source_container_name: The name of the ingress container that
-        is build from source. This could be empty string.
+      deploy_from_source_container_name: The name of the ingress container that
+        is deployed from source include build-from-source and zip deploy. This
+        field could be an empty string.
       build_worker_pool:  The name of the Cloud Build custom worker pool that
         should be used to build the function.
       build_machine_type: The machine type from Cloud Build default pool to use
@@ -868,7 +870,8 @@ class ServerlessOperations(object):
       enable_automatic_updates: If true, opt-in automatic build image updates.
         If false, opt-out automatic build image updates.
       is_verbose: Print verbose output. Forces polling instead of waiting.
-      source_bucket: The bucket to use for source uploads.
+      source_bucket: The existing bucket to use for source uploads. Leave it as
+        None to create a new bucket.
       kms_key: The KMS key to use for the deployment.
       iap_enabled: If true, assign run.invoker access to IAP P4SA, if false,
         remove run.invoker access from IAP P4SA.
@@ -878,7 +881,8 @@ class ServerlessOperations(object):
       service.Service, the service as returned by the server on the POST/PUT
        request to create/update the service.
     """
-    should_validate_service = build_source is not None and release_track in [
+    requires_build = build_source is not None and not skip_build
+    should_validate_service = requires_build and release_track in [
         base.ReleaseTrack.ALPHA,
         base.ReleaseTrack.BETA,
     ]
@@ -888,7 +892,8 @@ class ServerlessOperations(object):
           stages.ServiceStages(
               allow_unauthenticated is not None,
               include_validate_service=should_validate_service,
-              include_build=build_source is not None,
+              include_upload_source=build_source is not None,
+              include_build=requires_build,
               include_create_repo=repo_to_create is not None,
               include_iap=iap_enabled is not None,
           ),
@@ -898,25 +903,31 @@ class ServerlessOperations(object):
 
     if build_source is not None and skip_build:
       tracker.StartStage(stages.UPLOAD_SOURCE)
-      tracker.UpdateHeaderMessage('Uploading sources.')
-      source = sources.Upload(
-          build_source,
-          region,
-          service_ref,
-          source_bucket,
-          sources.ArchiveType.TAR,
-      )
-      # TODO(b/423646813): Remove this once zip deploys properly handles the
-      # generation number.
-      source.generation = None
-      source_path = sources.GetGsutilUri(source)
+      if sources.IsGcsObject(build_source):
+        tracker.UpdateHeaderMessage(
+            'Using the source from the specified bucket.'
+        )
+        source_path = build_source
+      else:
+        source = sources.Upload(
+            build_source,
+            region,
+            service_ref,
+            source_bucket,
+            sources.ArchiveType.TAR,
+            respect_gitignore=False,
+        )
+        # TODO(b/423646813): Remove this once zip deploys properly handles the
+        # generation number.
+        source.generation = None
+        source_path = sources.GetGsutilUri(source)
       config_changes.append(
           config_changes_mod.SourcesAnnotationChange(
-              updates={build_from_source_container_name: source_path}
+              updates={deploy_from_source_container_name: source_path}
           )
       )
       tracker.CompleteStage(stages.UPLOAD_SOURCE)
-    elif build_source is not None:
+    elif requires_build:
       new_conn = self._conn_context.GetContextWithRegionOverride(region)
       with new_conn:
         if should_validate_service:
@@ -965,7 +976,7 @@ class ServerlessOperations(object):
             build_image=build_image,
             build_name=build_name,
             build_base_image=build_base_image,
-            build_from_source_container_name=build_from_source_container_name,
+            build_from_source_container_name=deploy_from_source_container_name,
             enable_automatic_updates=enable_automatic_updates,
         )
         config_changes.append(_AddDigestToImageChange(image_digest))
@@ -991,8 +1002,10 @@ class ServerlessOperations(object):
           self._AddRevisionForcingChange(serv, config_changes)
         else:
           config_changes.append(_NewRevisionNonceChange(_Nonce()))
-        if serv and not with_image:
+        if serv and not with_image and not multiregion_regions:
           # Avoid changing the running code by making the new revision by digest
+          # We can't do this in multi-region services, because there is no
+          # Revisions API.
           self._EnsureImageDigest(serv, config_changes)
 
     if serv and serv.metadata.deletionTimestamp is not None:
@@ -1008,7 +1021,7 @@ class ServerlessOperations(object):
 
     # Handle SetIamPolicy call(s).
     self._HandleAllowUnauthenticated(
-        service_ref, allow_unauthenticated, allow_unauth_regions, tracker
+        service_ref, allow_unauthenticated, multiregion_regions, tracker
     )
 
     self._HandleIap(service_ref, iap_enabled, updated_service, tracker)
@@ -1016,7 +1029,6 @@ class ServerlessOperations(object):
     if not asyn and not dry_run:
       if updated_service.conditions.IsReady():
         return updated_service
-
       getter = (
           functools.partial(self.GetService, service_ref)
           if updated_service.operation_id is None or is_verbose
@@ -1025,7 +1037,7 @@ class ServerlessOperations(object):
       poller = op_pollers.ServiceConditionPoller(
           getter,
           tracker,
-          dependencies=stages.ServiceDependencies(),
+          dependencies=stages.ServiceDependencies(multiregion_regions),
           serv=updated_service,
       )
       self.WaitForCondition(poller)
@@ -1036,12 +1048,12 @@ class ServerlessOperations(object):
     return updated_service
 
   def _HandleAllowUnauthenticated(
-      self, service_ref, allow_unauthenticated, allow_unauth_regions, tracker
+      self, service_ref, allow_unauthenticated, multiregion_regions, tracker
   ):
     """Handle SetIamPolicy call(s)."""
-    if allow_unauthenticated is not None and allow_unauth_regions is not None:
+    if allow_unauthenticated is not None and multiregion_regions is not None:
       return self._HandleMultiRegionAllowUnauthenticated(
-          service_ref, allow_unauthenticated, allow_unauth_regions, tracker
+          service_ref, allow_unauthenticated, multiregion_regions, tracker
       )
     if allow_unauthenticated is not None:
       try:
@@ -1069,12 +1081,12 @@ class ServerlessOperations(object):
         )
 
   def _HandleMultiRegionAllowUnauthenticated(
-      self, service_ref, allow_unauthenticated, allow_unauth_regions, tracker
+      self, service_ref, allow_unauthenticated, multiregion_regions, tracker
   ):
     """Handle SetIamPolicy calls for Multi-Region Services."""
     tracker.StartStage(stages.SERVICE_IAM_POLICY_SET)
     warning_message = None
-    for region in allow_unauth_regions:
+    for region in multiregion_regions:
       try:
         tracker.UpdateStage(
             stages.SERVICE_IAM_POLICY_SET,
@@ -1108,12 +1120,27 @@ class ServerlessOperations(object):
           stages.SERVICE_IAM_POLICY_SET, warning_message=warning_message
       )
 
+  def GetServiceAgent(self, project_num):
+    """Returns the service agent for the given project.
+
+    For Universe Projects, the format will look like
+    service-{project_num}@gcp-sa-iap.{project_prefix}iam.gserviceaccount.com
+    FOR GDU format will look like
+    service-{project_num}@gcp-sa-iap.iam.gserviceaccount.com
+
+    Args:
+      project_num: The project number.
+    """
+    project_prefix = (
+        universe_descriptor.GetUniverseDomainDescriptor().project_prefix
+    )
+    if project_prefix:
+      return f'serviceAccount:service-{project_num}@gcp-sa-iap.{project_prefix}.iam.gserviceaccount.com'
+    return f'serviceAccount:service-{project_num}@gcp-sa-iap.iam.gserviceaccount.com'
+
   def _HandleIap(self, service_ref, iap_enabled, updated_service, tracker):
     """Handle IAP changes."""
-    iap_service_agent = (
-        'serviceAccount:service-%s@gcp-sa-iap.iam.gserviceaccount.com'
-        % updated_service.namespace
-    )
+    iap_service_agent = self.GetServiceAgent(updated_service.namespace)
     if iap_enabled is not None:
       try:
         tracker.StartStage(stages.SERVICE_IAP_ENABLE)
@@ -1350,135 +1377,6 @@ class ServerlessOperations(object):
     except api_exceptions.HttpError as e:
       exceptions.reraise(e)
 
-  # TODO(b/322180968): Once Worker API is ready, factor out worker related
-  # operations wired up to the API in a separate file.
-  def GetWorker(self, worker_ref):
-    return self.GetService(worker_ref)
-
-  def ReleaseWorker(
-      self,
-      worker_ref,
-      config_changes,
-      release_track,
-      tracker=None,
-      asyn=False,
-      for_replace=False,
-      prefetch=False,
-      build_image=None,
-      build_pack=None,
-      build_source=None,
-      repo_to_create=None,
-      already_activated_services=False,
-      dry_run=False,
-      generate_name=False,
-  ):
-    """Stubbed method for worker deploy surface.
-
-    Args:
-      worker_ref: Resource, the worker to release.
-      config_changes: list, objects that implement Adjust().
-      release_track: ReleaseTrack, the release track of a command calling this.
-      tracker: StagedProgressTracker, to report on the progress of releasing.
-      asyn: bool, if True, return without waiting for the service to be updated.
-      for_replace: bool, If the change is for a replacing the service from a
-        YAML specification.
-      prefetch: the service, pre-fetched for ReleaseService. `False` indicates
-        the caller did not perform a prefetch; `None` indicates a nonexistent
-        service.
-      build_image: The build image reference to the build.
-      build_pack: The build pack reference to the build.
-      build_source: The build source reference to the build.
-      repo_to_create: Optional
-        googlecloudsdk.command_lib.artifacts.docker_util.DockerRepo defining a
-        repository to be created.
-      already_activated_services: bool. If true, skip activation prompts for
-        services
-      dry_run: bool. If true, only validate the configuration.
-      generate_name: bool. If true, create a revision name, otherwise add nonce.
-
-    For private preview Worker is still Service underneath.
-
-    Returns:
-      service.Service, the service as returned by the server on the POST/PUT
-       request to create/update the service.
-    """
-    return self.ReleaseService(
-        worker_ref,
-        config_changes,
-        release_track,
-        tracker=tracker,
-        asyn=asyn,
-        for_replace=for_replace,
-        prefetch=prefetch,
-        build_image=build_image,
-        build_pack=build_pack,
-        build_source=build_source,
-        repo_to_create=repo_to_create,
-        already_activated_services=already_activated_services,
-        dry_run=dry_run,
-        generate_name=generate_name,
-    )
-
-  # TODO(b/322180968): Once Worker API is ready, replace Service related
-  # references.
-  def ListWorkers(self, namespace_ref):
-    """Returns all workers in the project/location specified."""
-    return self.ListServices(namespace_ref)
-
-  def DeleteWorker(self, worker_ref):
-    """Delete the provided Worker.
-
-    Args:
-      worker_ref: Resource, a reference to the Worker to delete
-
-    Raises:
-      WorkerNotFoundError: if provided worker is not found.
-    """
-    messages = self.messages_module
-    worker_name = worker_ref.RelativeName()
-    worker_delete_request = messages.RunNamespacesServicesDeleteRequest(
-        name=worker_name,
-    )
-
-    try:
-      with metrics.RecordDuration(metric_names.DELETE_SERVICE):
-        self._client.namespaces_services.Delete(worker_delete_request)
-    except api_exceptions.HttpNotFoundError:
-      raise serverless_exceptions.WorkerNotFoundError(
-          'Worker [{}] could not be found.'.format(worker_ref.servicesId)
-      )
-
-  def UpdateInstanceSplit(
-      self, worker_ref, config_changes, tracker, asyn, is_verbose
-  ):
-    return self.UpdateTraffic(
-        worker_ref, config_changes, tracker, asyn, is_verbose
-    )
-
-  def GetWorkerRevision(self, revision_ref):
-    return self.GetRevision(revision_ref)
-
-  def ListWorkerRevisions(
-      self, namespace_ref, worker_name, limit=None, page_size=100
-  ):
-    """List all worker revisions for the given worker.
-
-    Revision list gets sorted by worker name and creation timestamp.
-
-    Args:
-      namespace_ref: Resource, namespace to list worker revisions in
-      worker_name: str, The Worker for which to list revisions.
-      limit: Optional[int], max number of revisions to list.
-      page_size: Optional[int], number of revisions to fetch at a time
-
-    Returns:
-      Revisions for the given surface
-    """
-    return self.ListRevisions(namespace_ref, worker_name, limit, page_size)
-
-  def DeleteWorkerRevision(self, revision_ref):
-    return self.DeleteRevision(revision_ref)
-
   def ListExecutions(
       self, namespace_ref, label_selector='', limit=None, page_size=100
   ):
@@ -1957,13 +1855,12 @@ class ServerlessOperations(object):
     try:
       with metrics.RecordDuration(metric_names.GET_JOB):
         job_response = self._client.namespaces_jobs.Get(get_request)
+        return job.Job(job_response, messages)
     except api_exceptions.InvalidDataFromServerError as e:
       serverless_exceptions.MaybeRaiseCustomFieldMismatch(e)
     except api_exceptions.HttpNotFoundError:
       return None
 
-    return job.Job(job_response, messages)
-
   def GetTask(self, task_ref):
     """Return the relevant Task from the server, or None if 404."""
     messages = self.messages_module
@@ -1974,13 +1871,12 @@ class ServerlessOperations(object):
     try:
       with metrics.RecordDuration(metric_names.GET_TASK):
         task_response = self._client.namespaces_tasks.Get(get_request)
+        return task.Task(task_response, messages)
     except api_exceptions.InvalidDataFromServerError as e:
       serverless_exceptions.MaybeRaiseCustomFieldMismatch(e)
     except api_exceptions.HttpNotFoundError:
       return None
 
-    return task.Task(task_response, messages)
-
   def GetExecution(self, execution_ref):
     """Return the relevant Execution from the server, or None if 404."""
     messages = self.messages_module
@@ -1991,13 +1887,12 @@ class ServerlessOperations(object):
     try:
       with metrics.RecordDuration(metric_names.GET_EXECUTION):
         execution_response = self._client.namespaces_executions.Get(get_request)
+        return execution.Execution(execution_response, messages)
     except api_exceptions.InvalidDataFromServerError as e:
       serverless_exceptions.MaybeRaiseCustomFieldMismatch(e)
     except api_exceptions.HttpNotFoundError:
       return None
 
-    return execution.Execution(execution_response, messages)
-
   def ListJobs(self, namespace_ref):
     """Returns all jobs in the namespace."""
     messages = self.messages_module
@@ -2007,11 +1902,10 @@ class ServerlessOperations(object):
     try:
       with metrics.RecordDuration(metric_names.LIST_JOBS):
         response = self._client.namespaces_jobs.List(request)
+        return [job.Job(item, messages) for item in response.items]
     except api_exceptions.InvalidDataFromServerError as e:
       serverless_exceptions.MaybeRaiseCustomFieldMismatch(e)
 
-    return [job.Job(item, messages) for item in response.items]
-
   def DeleteJob(self, job_ref):
     """Delete the provided Job.
 
